@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.147 2008/05/22 13:20:35 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.149 2008/11/27 05:56:12 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -1222,11 +1222,12 @@ FileCopyPeerFA::FileCopyPeerFA(const ParsedURL *u,int m)
       SetError(_("file name missed in URL"));
 }
 
-FileCopyPeerFA::~FileCopyPeerFA()
+void FileCopyPeerFA::PrepareToDie()
 {
    if(session)
       session->Close();
 }
+FileCopyPeerFA::~FileCopyPeerFA() {}
 
 FileCopyPeerFA *FileCopyPeerFA::New(FileAccess *s,const char *url,int m)
 {
@@ -1776,8 +1777,6 @@ int FileCopyPeerDirList::Do()
 void FileVerificator::Init0()
 {
    done=false;
-   verify_buffer=0;
-   verify_process=0;
    if(!ResMgr::QueryBool("xfer:verify",0))
       done=true;
 }
@@ -1789,7 +1788,7 @@ void FileVerificator::InitVerify(const char *f)
    args->Append(f);
    verify_process=new InputFilter(args);
    verify_process->StderrToStdout();
-   verify_buffer=new IOBufferFDStream(verify_process,IOBuffer::GET);
+   verify_buffer=new IOBufferFDStream(verify_process.Cast<FDStream>(),IOBuffer::GET);
 }
 FileVerificator::FileVerificator(const char *f)
 {
@@ -1837,11 +1836,9 @@ FileVerificator::FileVerificator(const FileAccess *session,const char *f)
    InitVerify(f);
    verify_process->SetCwd(session->GetCwd());
 }
-FileVerificator::~FileVerificator()
-{
-   // verify_process is deleted by verify_buffer dtor
-   Delete(verify_buffer);
-}
+
+FileVerificator::~FileVerificator() {}
+
 int FileVerificator::Do()
 {
    int m=STALL;
