@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,12 +18,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopyFtp.cc,v 1.22 2005/03/09 07:54:09 lav Exp $ */
+/* $Id: FileCopyFtp.cc,v 1.27 2008/11/27 05:56:12 lav Exp $ */
 
 #include <config.h>
 
-#include <getopt.h>
-
 #include "FileCopyFtp.h"
 #include "log.h"
 
@@ -34,6 +32,9 @@
 #define passive_ssl_connect true
 #endif
 
+#define ftp_src get->GetSession().Cast<Ftp>()
+#define ftp_dst put->GetSession().Cast<Ftp>()
+
 void FileCopyFtp::Close()
 {
    ftp_src->Close();
@@ -54,7 +55,7 @@ int FileCopyFtp::Do()
 	 return m;
       if(ftp_dst->IsClosed())
       {
-	 ((FileCopyPeerFA*)put)->OpenSession();
+	 put->OpenSession();
 	 ftp_dst->SetCopyMode(Ftp::COPY_DEST,!passive_source,protect,
 	       passive_source^passive_ssl_connect,dst_retries,dst_try_time);
 	 m=MOVED;
@@ -70,14 +71,14 @@ int FileCopyFtp::Do()
 
    if(ftp_src->IsClosed())
    {
-      ((FileCopyPeerFA*)get)->OpenSession();
+      get->OpenSession();
       ftp_src->SetCopyMode(Ftp::COPY_SOURCE,passive_source,protect,
 	    !passive_source^passive_ssl_connect,src_retries,src_try_time);
       m=MOVED;
    }
    if(ftp_dst->IsClosed())
    {
-      ((FileCopyPeerFA*)put)->OpenSession();
+      put->OpenSession();
       ftp_dst->SetCopyMode(Ftp::COPY_DEST,!passive_source,protect,
 	    passive_source^passive_ssl_connect,dst_retries,dst_try_time);
       m=MOVED;
@@ -118,8 +119,8 @@ int FileCopyFtp::Do()
 	 Log::global->Write(0,_("**** FXP: giving up, reverting to plain copy\n"));
 	 Close();
 	 disable_fxp=true;
-	 ((FileCopyPeerFA*)get)->SetFXP(false);
-	 ((FileCopyPeerFA*)put)->SetFXP(false);
+	 get->SetFXP(false);
+	 put->SetFXP(false);
 
 	 if(ResMgr::QueryBool("ftp:fxp-force",ftp_src->GetHostName())
 	 || ResMgr::QueryBool("ftp:fxp-force",ftp_dst->GetHostName()))
@@ -186,9 +187,11 @@ int FileCopyFtp::Do()
       RateReset();
    }
 
-   // check for timeout when target is done, and source is stale
+   // check for timeout when one session is done, and the other is stuck
    if(dst_res==FA::OK && src_res==FA::IN_PROGRESS)
       ftp_src->CopyCheckTimeout(ftp_dst);
+   if(src_res==FA::OK && dst_res==FA::IN_PROGRESS)
+      ftp_dst->CopyCheckTimeout(ftp_src);
 
    off_t add=ftp_dst->GetPos()-put->GetRealPos();
    if(add>0)
@@ -206,7 +209,6 @@ int FileCopyFtp::Do()
 
 void FileCopyFtp::Init()
 {
-   ftp_src=ftp_dst=0;
    no_rest=false;
    orig_passive_source=passive_source=false;
    src_retries=dst_retries=0;
@@ -218,22 +220,15 @@ void FileCopyFtp::Init()
 #endif
 }
 
-FileCopyFtp::~FileCopyFtp()
-{
-}
-
 // s,d must be FileCopyPeerFA, s->GetSession(),d->GetSession() must be Ftp.
 FileCopyFtp::FileCopyFtp(FileCopyPeer *s,FileCopyPeer *d,bool c,bool rp)
    : super(s,d,c)
 {
    Init();
    passive_source=rp;
-   // not pretty.
-   ftp_src=(Ftp*)(s->GetSession());
-   ftp_dst=(Ftp*)(d->GetSession());
 
-   ((FileCopyPeerFA*)s)->SetFXP(true);
-   ((FileCopyPeerFA*)d)->SetFXP(true);
+   get->SetFXP(true);
+   put->SetFXP(true);
 
    if(ftp_src->IsPassive() && !ftp_dst->IsPassive())
       passive_source=true;
@@ -252,8 +247,8 @@ FileCopyFtp::FileCopyFtp(FileCopyPeer *s,FileCopyPeer *d,bool c,bool rp)
 
 FileCopy *FileCopyFtp::New(FileCopyPeer *s,FileCopyPeer *d,bool c)
 {
-   FA *s_s=s->GetSession();
-   FA *d_s=d->GetSession();
+   const FileAccessRef& s_s=s->GetSession();
+   const FileAccessRef& d_s=d->GetSession();
    if(!s_s || !d_s)
       return 0;
    if((strcmp(s_s->GetProto(),"ftp") && strcmp(s_s->GetProto(),"ftps"))
