@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.61 2007/10/09 12:49:41 lav Exp $ */
+/* $Id: FileSet.cc,v 1.67 2008/11/27 05:56:14 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -137,6 +137,10 @@ FileSet::FileSet()
 {
 }
 
+FileSet::~FileSet()
+{
+}
+
 /* we don't copy the sort state--nothing needs it, and it'd
  * be a bit of a pain to implement. */
 FileSet::FileSet(FileSet const *set)
@@ -645,6 +649,15 @@ FileInfo::FileInfo(const FileInfo &fi)
    nlinks=fi.nlinks;
    longname.set(fi.longname);
 }
+FileInfo::FileInfo(const char *n)
+{
+   Init();
+   SetName(n);
+}
+
+FileInfo::~FileInfo()
+{
+}
 
 #ifndef S_ISLNK
 # define S_ISLNK(mode) (S_IFLNK==(mode&S_IFMT))
@@ -806,7 +819,9 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    t = NEXT_TOKEN;
    if(!t)
       ERR;
+   date.tm_isdst=-1;
    date.tm_hour=date.tm_min=0;
+   date.tm_sec=30;
    int prec=30;
    if(strlen(t)==5)
    {
@@ -822,12 +837,10 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
       /* We don't know the hour.  Set it to something other than 0, or
        * DST -1 will end up changing the date. */
       date.tm_hour = 12;
-      prec=12*60*60;
+      date.tm_sec=0;
+      prec=12*HOUR;
    }
 
-   date.tm_isdst=-1;
-   date.tm_sec=30;
-
    fi->SetDate(mktime_from_tz(&date,tz),prec);
 
    char *name=strtok_r(NULL,"",&next);
