@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.76 2008/01/21 14:07:00 lav Exp $ */
+/* $Id: SFtp.cc,v 1.82 2008/11/27 05:56:27 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -35,7 +35,7 @@
 
 #define max_buf 0x10000
 
-#define super NetAccess
+#define super SSH_Access
 
 bool SFtp::GetBetterConnection(int level,bool limit_reached)
 {
@@ -95,7 +95,8 @@ int SFtp::Do()
       Disconnect();
       return MOVED;
    }
-   m|=HandleReplies();
+   if(state!=CONNECTING_1)
+      m|=HandleReplies();
 
    if(Error())
       return m;
@@ -200,18 +201,15 @@ int SFtp::Do()
 	 TimeoutS(1);
 	 return m;
       }
-      int pipe_in =ssh->getfd_pipe_in();
-      int pipe_out=ssh->getfd_pipe_out();
-      ssh->Kill(SIGCONT);
-      pty_send_buf=new IOBufferFDStream(ssh.borrow(),IOBuffer::PUT);
-      pty_recv_buf=new IOBufferFDStream(new FDStream(fd,"pseudo-tty"),IOBuffer::GET);
-      send_buf=new IOBufferFDStream(new FDStream(pipe_out,"pipe-out"),IOBuffer::PUT);
-      recv_buf=new IOBufferFDStream(new FDStream(pipe_in,"pipe-in"),IOBuffer::GET);
+      MakePtyBuffers();
       set_real_cwd("~");
       state=CONNECTING_1;
       m=MOVED;
    }
    case CONNECTING_1:
+      m|=HandleSSHMessage();
+      if(state!=CONNECTING_1)
+	 return MOVED;
       if(!received_greeting)
 	 return m;
       SendRequest(new Request_INIT(Query("protocol-version",hostname)),Expect::FXP_VERSION);
@@ -219,6 +217,9 @@ int SFtp::Do()
       return MOVED;
 
    case CONNECTING_2:
+      m|=HandleSSHMessage();
+      if(state!=CONNECTING_2)
+	 return MOVED;
       if(protocol_version==0)
 	 return m;
       if(home_auto==0)
@@ -295,13 +296,10 @@ int SFtp::Do()
 
 void SFtp::MoveConnectionHere(SFtp *o)
 {
+   super::MoveConnectionHere(o);
    protocol_version=o->protocol_version;
    recv_translate=o->recv_translate.borrow();
    send_translate=o->send_translate.borrow();
-   send_buf=o->send_buf.borrow();
-   recv_buf=o->recv_buf.borrow();
-   pty_send_buf=o->pty_send_buf.borrow();
-   pty_recv_buf=o->pty_recv_buf.borrow();
    rate_limit=o->rate_limit.borrow();
    expect_queue_size=o->expect_queue_size; o->expect_queue_size=0;
    expect_chain=o->expect_chain; o->expect_chain=0;
@@ -320,21 +318,13 @@ void SFtp::MoveConnectionHere(SFtp *o)
 
 void SFtp::Disconnect()
 {
-   if(send_buf)
-      LogNote(9,_("Disconnecting"));
+   super::Disconnect();
    handle.set(0);
-   send_buf=0;
-   recv_buf=0;
-   pty_send_buf=0;
-   pty_recv_buf=0;
    file_buf=0;
-   ssh=0;
    EmptyRespQueue();
    state=DISCONNECTED;
    if(mode==STORE)
       SetError(STORE_FAILED);
-   received_greeting=false;
-   password_sent=0;
    protocol_version=0;
    send_translate=0;
    recv_translate=0;
@@ -364,7 +354,7 @@ void SFtp::Init()
    flush_timer.Set(0,500);
 }
 
-SFtp::SFtp()
+SFtp::SFtp() : SSH_Access("SFTP:")
 {
    Init();
    Reconfig(0);
@@ -398,7 +388,12 @@ void SFtp::Packet::PackString(Buffer *b,const char *str,int len)
 SFtp::unpack_status_t SFtp::Packet::UnpackString(const Buffer *b,int *offset,int limit,xstring *str_out)
 {
    if(limit-*offset<4)
-      return b->Eof()?UNPACK_PREMATURE_EOF:UNPACK_NO_DATA_YET;
+   {
+      // We unpack strings when we have already received complete packet,
+      // so it is not possible to receive any more data.
+      LogError(2,"bad string in reply (truncated length field)");
+      return UNPACK_WRONG_FORMAT;
+   }
 
    int len=b->UnpackUINT32BE(*offset);
    if(len>limit-*offset-4)
@@ -451,8 +446,8 @@ SFtp::unpack_status_t SFtp::Packet::Unpack(const Buffer *b)
 
 SFtp::unpack_status_t SFtp::UnpackPacket(Buffer *b,SFtp::Packet **p)
 {
-   *p=0;
    Packet *&pp=*p;
+   pp=0;
 
    Packet probe;
    unpack_status_t res=probe.Unpack(b);
@@ -527,8 +522,8 @@ SFtp::unpack_status_t SFtp::UnpackPacket(Buffer *b,SFtp::Packet **p)
 	 ;
       }
       probe.DropData(b);
-      delete *p;
-      *p=0;
+      delete pp;
+      pp=0;
    }
    return res;
 }
@@ -576,6 +571,7 @@ void SFtp::SendRequest()
    switch((open_mode)mode)
    {
    case CHANGE_DIR:
+      LogNote(9,"checking directory `%s'",file.get());
       SendRequest(new Request_STAT(lc_to_utf8(file),0,protocol_version),Expect::CWD);
       SendRequest(new Request_STAT(lc_to_utf8(dir_file(file,".")),0,protocol_version),Expect::CWD);
       state=WAITING;
@@ -712,37 +708,6 @@ int SFtp::HandlePty()
    const char *eol=(const char*)memchr(b,'\n',s);
    if(!eol)
    {
-      const char *p="password:";
-      const char *y="(yes/no)?";
-      int p_len=strlen(p);
-      int y_len=strlen(y);
-      if(s>0 && b[s-1]==' ')
-	 s--;
-      if(s>=p_len && !strncasecmp(b+s-p_len,p,p_len)
-      || s>10 && !strncmp(b+s-2,"':",2))
-      {
-	 if(!pass)
-	 {
-	    SetError(LOGIN_FAILED,_("Password required"));
-	    return MOVED;
-	 }
-	 if(password_sent>1)
-	 {
-	    SetError(LOGIN_FAILED,_("Login incorrect"));
-	    return MOVED;
-	 }
-	 pty_recv_buf->Put("XXXX");
-	 pty_send_buf->Put(pass);
-	 pty_send_buf->Put("\n");
-	 password_sent++;
-	 return m;
-      }
-      if(s>=y_len && !strncasecmp(b+s-y_len,y,y_len))
-      {
-	 pty_recv_buf->Put("yes\n");
-	 pty_send_buf->Put("yes\n");
-	 return m;
-      }
       if(pty_recv_buf->Eof())
 	 LogError(0,_("Peer closed connection"));
       if(pty_recv_buf->Error())
@@ -762,8 +727,6 @@ int SFtp::HandlePty()
    pty_recv_buf->Skip(s);
 
    LogRecv(4,line);
-   if(!received_greeting && !strcmp(line,"SFTP:"))
-      received_greeting=true;
 
    return m;
 }
@@ -771,6 +734,13 @@ int SFtp::HandlePty()
 void SFtp::HandleExpect(Expect *e)
 {
    const Packet *reply=e->reply;
+   if(reply->TypeIs(SSH_FXP_STATUS))
+   {
+      Reply_STATUS *r=(Reply_STATUS*)reply;
+      const char *message=r->GetMessage();
+      LogNote(9,"status code=%d(%s), message=%s",r->GetCode(),r->GetCodeText(),
+	 message?message:"NULL");
+   }
    switch(e->tag)
    {
    case Expect::FXP_VERSION:
@@ -961,29 +931,29 @@ void SFtp::HandleExpect(Expect *e)
       }
       break;
    case Expect::INFO:
+      entity_size=NO_SIZE;
+      entity_date=NO_DATE;
       if(reply->TypeIs(SSH_FXP_ATTRS))
       {
 	 const FileAttrs *a=((Reply_ATTRS*)reply)->GetAttrs();
-	 entity_size=NO_SIZE;
-	 entity_date=NO_DATE;
 	 if(a->flags&SSH_FILEXFER_ATTR_SIZE)
 	    entity_size=a->size;
 	 if(a->flags&SSH_FILEXFER_ATTR_MODIFYTIME)
 	    entity_date=a->mtime;
-	 if(mode==ARRAY_INFO)
-	 {
-	    array_for_info[e->i].size=entity_size;
-	    array_for_info[e->i].get_size=false;
-	    array_for_info[e->i].time=entity_date;
-	    array_for_info[e->i].get_time=false;
-	    break;
-	 }
 	 LogNote(9,"file info: size=%lld, date=%s",(long long)entity_size,ctime(&entity_date));
-	 if(opt_size)
-	    *opt_size=entity_size;
-	 if(opt_date)
-	    *opt_date=entity_date;
       }
+      if(mode==ARRAY_INFO)
+      {
+	 array_for_info[e->i].size=entity_size;
+	 array_for_info[e->i].get_size=false;
+	 array_for_info[e->i].time=entity_date;
+	 array_for_info[e->i].get_time=false;
+	 break;
+      }
+      if(opt_size)
+	 *opt_size=entity_size;
+      if(opt_date)
+	 *opt_date=entity_date;
       break;
    case Expect::WRITE_STATUS:
       if(reply->TypeIs(SSH_FXP_STATUS))
@@ -1023,10 +993,13 @@ void SFtp::RequestMoreData()
 
 int SFtp::HandleReplies()
 {
-   int m=HandlePty();
+   int m=STALL;
    if(recv_buf==0)
       return m;
 
+   if(state!=CONNECTING_2)
+      m|=HandlePty();
+
    int i=0;
    Expect *ooo_scan=ooo_chain;
    while(ooo_scan)
@@ -1883,9 +1856,19 @@ SFtp::unpack_status_t SFtp::Reply_STATUS::Unpack(const Buffer *b)
    UNPACK32(code);
    if(protocol_version>=3)
    {
+      if(unpacked>=limit)
+      {
+	 LogError(2,"Status reply lacks `error message' field");
+	 return UNPACK_SUCCESS;
+      }
       res=Packet::UnpackString(b,offset,limit,&message);
       if(res!=UNPACK_SUCCESS)
 	 return res;
+      if(unpacked>=limit)
+      {
+	 LogError(2,"Status reply lacks `language tag' field");
+	 return UNPACK_SUCCESS;
+      }
       res=Packet::UnpackString(b,offset,limit,&language);
       if(res!=UNPACK_SUCCESS)
 	 return res;
@@ -1929,7 +1912,7 @@ void SFtp::Request_RENAME::Pack(Buffer *b)
 
 const char *SFtp::utf8_to_lc(const char *s)
 {
-   if(!recv_translate)
+   if(!recv_translate || !s)
       return s;
 
    recv_translate->ResetTranslation();
@@ -1942,7 +1925,7 @@ const char *SFtp::utf8_to_lc(const char *s)
 }
 const char *SFtp::lc_to_utf8(const char *s)
 {
-   if(!send_translate)
+   if(!send_translate || !s)
       return s;
 
    send_translate->ResetTranslation();
@@ -1954,10 +1937,20 @@ const char *SFtp::lc_to_utf8(const char *s)
    return s;
 }
 
+FileSet *SFtp::GetFileSet()
+{
+   FileSet *fset=file_set.borrow();
+   return fset?fset:new FileSet;
+}
+
 FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
 {
    const FileAttrs *a=&na->attrs;
    const char *name=utf8_to_lc(na->name);
+   const char *longname=utf8_to_lc(na->longname);
+
+   LogNote(10,"NameAttrs(name=\"%s\",type=%d,longname=\"%s\")\n",name?name:"",a->type,longname?longname:"");
+
    if(!name || !name[0])
       return 0;
    if(strchr(name,'/'))
@@ -1970,10 +1963,11 @@ FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
    case SSH_FILEXFER_TYPE_REGULAR:  fi->SetType(fi->NORMAL);    break;
    case SSH_FILEXFER_TYPE_DIRECTORY:fi->SetType(fi->DIRECTORY); break;
    case SSH_FILEXFER_TYPE_SYMLINK:  fi->SetType(fi->SYMLINK);   break;
+   case SSH_FILEXFER_TYPE_UNKNOWN: break;
    default: return 0;
    }
-   if(na->longname)
-      fi->SetLongName(utf8_to_lc(na->longname));
+   if(longname)
+      fi->SetLongName(longname);
    if(a->flags&SSH_FILEXFER_ATTR_SIZE)
       fi->SetSize(a->size);
    if(a->flags&SSH_FILEXFER_ATTR_UIDGID)
