@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -20,7 +20,6 @@
 
 #include <config.h>
 #include "buffer_ssl.h"
-#include "xmalloc.h"
 
 #if USE_SSL
 # include "lftp_ssl.h"
@@ -39,12 +38,11 @@ int IOBufferSSL::Do()
    switch(mode)
    {
    case PUT:
-      if(in_buffer==0 && ssl->handshake_done)
+      if(Size()==0 && ssl->handshake_done)
 	 return STALL;
-      res=Put_LL(buffer+buffer_ptr,in_buffer);
+      res=Put_LL(buffer+buffer_ptr,Size());
       if(res>0)
       {
-	 in_buffer-=res;
 	 buffer_ptr+=res;
 	 event_time=now;
 	 return MOVED;
@@ -82,8 +80,7 @@ int IOBufferSSL::Do()
 
 int IOBufferSSL::Get_LL(int size)
 {
-   Allocate(size);
-   int res=ssl->read(buffer+buffer_ptr+in_buffer,size);
+   int res=ssl->read(GetSpace(size),size);
    if(res<0)
    {
       if(res==ssl->RETRY)
@@ -115,16 +112,14 @@ int IOBufferSSL::Put_LL(const char *buf,int size)
    return res;
 }
 
-IOBufferSSL::IOBufferSSL(lftp_ssl *s,dir_t m)
- : IOBuffer(m)
+int IOBufferSSL::PutEOF_LL()
 {
-   ssl=s;
-   close_later=false;
+   ssl->shutdown();
+   return 0;
 }
+
 IOBufferSSL::~IOBufferSSL()
 {
-   if(close_later)
-      delete ssl;
 }
 
 #endif // USE_SSL
