@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.285 2008/01/21 14:07:01 lav Exp $ */
+/* $Id: commands.cc,v 1.291 2008/11/27 05:56:35 lav Exp $ */
 
 #include <config.h>
 
@@ -31,6 +31,7 @@
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <assert.h>
 #ifdef HAVE_DLFCN_H
@@ -76,11 +77,7 @@
 #include "LocalDir.h"
 #include "ConnectionSlot.h"
 
-#include "confpaths.h"
-
-#define MINUTE (60)
-#define HOUR   (60*MINUTE)
-#define DAY    (24*HOUR)
+#include "configmake.h"
 
 History	 cwd_history;
 
@@ -360,6 +357,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -e <cmd>            execute the command just after selecting\n"
 	 " -u <user>[,<pass>]  use the user/password for authentication\n"
 	 " -p <port>           use the port for connection\n"
+	 " -s <slot>           assign the connection to this slot\n"
 	 " <site>              host name, URL or bookmark name\n")},
    {"pget",    cmd_get,    N_("pget [OPTS] <rfile> [-o <lfile>]"),
 	 N_("Gets the specified file using several connections. This can speed up transfer,\n"
@@ -453,7 +451,10 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "Sleep for given amount of time. The time argument can be optionally\n"
 	 "followed by unit specifier: d - days, h - hours, m - minutes, s - seconds.\n"
 	 "By default time is assumed to be seconds.\n")},
-   {"slot",    cmd_slot,   0,0},
+   {"slot",    cmd_slot, 0,
+        N_("Usage: slot [<label>]\n"
+   	"List assigned slots.\n"
+	"If <label> is specified, switch to the slot named <label>.\n")},
    {"source",  cmd_source, N_("source <file>"),
 	 N_("Execute commands recorded in file <file>\n")},
    {"suspend", cmd_suspend},
@@ -642,6 +643,7 @@ Job *CmdExec::builtin_cd()
 
 Job *CmdExec::builtin_exit()
 {
+   bool detach=ResMgr::QueryBool("cmd:move-background-detach",0);
    bool bg=false;
    bool kill=false;
    int code=prev_exit_code;
@@ -670,7 +672,7 @@ Job *CmdExec::builtin_exit()
    }
    // Note: one job is this CmdExec.
    if(!bg && exec->top_level
-   && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfJobs()>1)
+   && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfJobs()>0)
    {
       eprintf(_(
 	 "There are running jobs and `cmd:move-background' is not set.\n"
@@ -678,9 +680,37 @@ Job *CmdExec::builtin_exit()
       ));
       return 0;
    }
+   if(!detach && Job::NumberOfJobs()==0)
+      detach=true;
    if(kill)
       Job::KillAll();
-   exec->Exit(code);
+   else if(detach)
+      exec->Exit(code);
+   else {
+      int loc=0;
+      exec->SetAutoTerminateInBackground(true);
+      eprintf(_(
+	 "\n"
+	 "lftp now tricks the shell to move it to background process group.\n"
+	 "lftp continues to run in the background despite the `Stopped' message.\n"
+	 "lftp will automatically terminate when all jobs are finished.\n"
+	 "Use `fg' shell command to return to lftp if it is still running.\n"
+      ));
+      // trick the shell
+      switch(pid_t pid=fork()) {
+      case 0: // child
+	 sleep(1);   // wait for the parent to stop (is there a safer way?)
+	 ::kill(getppid(),SIGCONT);
+	 _exit(0);
+      default: // parent
+	 raise(SIGSTOP);
+	 waitpid(pid,&loc,0); // clean-up
+	 break;
+      case -1:
+	 exec->Exit(code);
+	 break;
+      }
+   }
    exit_code=code;
    return 0;
 }
@@ -791,14 +821,18 @@ Job *CmdExec::builtin_open()
       {"execute",required_argument,0,'e'},
       {"debug",optional_argument,0,'d'},
       {"no-bookmark",no_argument,0,'B'},
+      {"slot",required_argument,0,'s'},
       {"help",no_argument,0,'h'},
       {0,0,0,0}
    };
 
-   while((c=args->getopt_long("u:p:e:dBh",open_options))!=EOF)
+   while((c=args->getopt_long("u:p:e:s:dBh",open_options))!=EOF)
    {
       switch(c)
       {
+      case('s'):
+      	if (*optarg) ChangeSlot(optarg);
+        break;
       case('p'):
 	 port=optarg;
 	 break;
@@ -2539,12 +2573,21 @@ CMD(ver)
       _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2008);
    printf("\n");
    printf(
-      _("LFTP is free software, covered by the GNU General Public License, and you are\n"
-      "welcome to change it and/or distribute copies of it under certain conditions.\n"
-      "There is absolutely no warranty for LFTP.  See COPYING for details.\n"));
+      _("LFTP is free software: you can redistribute it and/or modify\n"
+      "it under the terms of the GNU General Public License as published by\n"
+      "the Free Software Foundation, either version 3 of the License, or\n"
+      "(at your option) any later version.\n"
+      "\n"
+      "This program is distributed in the hope that it will be useful,\n"
+      "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+      "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+      "GNU General Public License for more details.\n"
+      "\n"
+      "You should have received a copy of the GNU General Public License\n"
+      "along with LFTP.  If not, see <http://www.gnu.org/licenses/>.\n"));
    printf("\n");
    printf(
-      _("Send bug reports and questions to <%s>.\n"),"lftp@uniyar.ac.ru");
+      _("Send bug reports and questions to the mailing list <%s>.\n"),"lftp@uniyar.ac.ru");
 
 #if defined(HAVE_DLOPEN) && defined(RTLD_DEFAULT)
    /* Show some of loaded libraries. Modules can load those libraries on
