@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,13 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: history.cc,v 1.16 2005/12/27 06:25:50 lav Exp $ */
+/* $Id: history.cc,v 1.22 2008/11/27 05:56:37 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <stddef.h>
 #include "trio.h"
 #include "history.h"
 #include "url.h"
@@ -38,14 +39,9 @@ History::History()
    stamp=0;
    fd=-1;
    modified=false;
-   file=0;
    const char *home=get_lftp_home();
    if(home)
-   {
-      const char *add="/cwd_history";
-      file=xstrdup(home,+strlen(add));
-      strcat(file,add);
-   }
+      file.vset(home,"/cwd_history",NULL);
 }
 
 History::~History()
@@ -53,29 +49,20 @@ History::~History()
    Close();
    if(full)
       delete full;
-   xfree(file);
 }
 
 const char *History::extract_url(const char *res)
 {
-   static char *buf;
-   static int buf_len;
-
    const char *url=strchr(res,':');
    if(url)
       url++;
    else
       url=res;
 
-   int len=strlen(url);
-   if(len>=buf_len);
-      buf=(char*)xrealloc(buf,buf_len=len+64);
+   if(url::is_url(url))
+      return url;
 
-   strcpy(buf,url);
-   if(!url::is_url(buf))
-      url::decode_string(buf);
-
-   return buf;
+   return url::decode(url);
 }
 
 time_t History::extract_stamp(const char *res)
@@ -83,7 +70,7 @@ time_t History::extract_stamp(const char *res)
    return atol(res);
 }
 
-const char *History::Lookup(FileAccess *s)
+const char *History::Lookup(const FileAccess *s)
 {
    const char *url=s->GetConnectURL(s->NO_PATH|s->NO_PASSWORD);
    if(!url)
@@ -126,7 +113,7 @@ void History::Load()
 	 return;
       fcntl(fd,F_SETFD,FD_CLOEXEC);
       if(Lock(fd,F_RDLCK)==-1)
-	 fprintf(stderr,"%s: lock for reading failed, trying to read anyway\n",file);
+	 fprintf(stderr,"%s: lock for reading failed, trying to read anyway\n",file.get());
    }
    if(!full)
       full=new KeyValueDB;
@@ -147,7 +134,7 @@ void History::Close()
    }
 }
 
-void History::Set(FileAccess *s,const FileAccess::Path &cwd)
+void History::Set(const FileAccess *s,const FileAccess::Path &cwd)
 {
    if(cwd.path==0 || !strcmp(cwd.path,"~") || s->GetHostName()==0)
       return;
@@ -178,7 +165,7 @@ void History::Save()
    fcntl(fd,F_SETFD,FD_CLOEXEC);
    if(Lock(fd,F_WRLCK)==-1)
    {
-      fprintf(stderr,"%s: lock for writing failed\n",file);
+      fprintf(stderr,"%s: lock for writing failed\n",file.get());
       Close();
       return;
    }
