@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,37 +18,38 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mgetJob.h,v 1.6 2001/12/05 15:03:45 lav Exp $ */
+/* $Id: mgetJob.h,v 1.13 2008/11/27 05:56:40 lav Exp $ */
 
 #ifndef MGETJOB_H
 #define MGETJOB_H
 
 #include "GetJob.h"
-#include "Glob.h"
+#include "FileGlob.h"
+#include "mkdirJob.h"
 
 class mgetJob : public GetJob
 {
-   GlobURL *rg;
-   class mkdirJob *mkdir_job;
-   ArgV *mkdir_args;
+   Ref<GlobURL> rg;
+   JobRef<mkdirJob> mkdir_job;
+   Ref<ArgV> mkdir_args;
    int mkdir_base_arg;
-   ArgV *m_args;
-   char *output_dir;
+   Ref<ArgV> m_args;
+   xstring_c output_dir;
 
    bool	 make_dirs;
-   void	 make_directory(char *d);
+   void	 make_directory(const char *d);
 
    void LocalGlob(const char *p);
 
 public:
    int	 Do();
-   void	 PrintStatus(int);
-   void	 ShowRunStatus(StatusLine *s);
+   void	 PrintStatus(int,const char *);
+   void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
 
    mgetJob(FileAccess *session,ArgV *args,bool c,bool md);
    ~mgetJob();
 
-   void OutputDir(char *o) { output_dir=o; }
+   void OutputDir(char *o) { output_dir.set_allocated(o); }
 };
 
 #endif // MGETJOB_H
